/* UNH CS 758 assignment 7

   Parse a string using a context-free grammar.

   see parse.h for overview of major data structures
*/

#include <stdio.h>
#include <stdlib.h>  /* free, EXIT_SUCCESS, rand */
#include <string.h>  /* strtok */
#include <ctype.h>   /* islower */
#include "utils.h"
#include "parse.h"


/************************************************************
 * arg parsing and main
 ************************************************************/


void usage(char *const argv[])
{
  fprintf(stderr, "usage: %s <grammarfile> <algname>\n", argv[0]);
  fprintf(stderr,
	  "where <grammarfile> is the path of a file containing a CFG in CNF\n"
	  "  and <algname> is either enum or cyk\n"
	  "The sentence to parse is read from stdin and a parse is written to stdout.\n");
  exit(EXIT_FAILURE);
}


int main(int argc, char *const argv[])
{
  FILE *infile;
  cfg *g;
  sentence *s;
  parse *ps;

  if (argc != 3) usage(argv);
  infile = fopen(argv[1], "r");
  if (!infile) panic ("can't open the specified grammar file");
  g = read_grammar(infile);
  /* fprint_grammar(stderr, g); */
  if (fclose(infile) != 0) panic("error closing grammar file");
  s = read_sentence(stdin, g);
  /* fprint_sentence(stderr, s); */
  if(strcmp(argv[2], "enum") == 0)
    ps = enum_parse(s, g);
  else if(strcmp(argv[2], "cyk") == 0)
    ps = cyk_most_probable_parse(s, g);
  else
    usage(argv);
  fprint_parses(stdout, ps, s);
  free_parses_deep(ps);
  free_sentence(s);
  free_grammar(g);
  return EXIT_SUCCESS;
}


/* EOF */
