/* UNH CS 758 assignment 7

   Parse a string using a context-free grammar.

   written in haste by Wheeler Ruml, October 2024


   possible improvements:
     remove intrusive linked list from parse, make separate
     write enumerative parser
     flag to print all or only one parse
     verbose flag to print debugging info
     check grammar for reasonableness
*/



#include <stdio.h>

#define START_NONTERMINAL_NAME "S"


/* context-free grammar (CFG) in Chomsky normal form (CNF)

   two types of rules: non-terminal -> non-terminal-1 non-terminal-2
                       non-terminal -> terminal

   a grammer owns all the strings in its rules

   grammar rules are stored in a singly-linked list
*/


typedef struct terminal_rule {
  char *nonterminal;
  char *terminal;
  double probability;
  struct terminal_rule *next;
} terminal_rule;


typedef struct nonterminal_rule {
  char *nonterminal;
  char *nonterminal_1;
  char *nonterminal_2;
  double probability;
  struct nonterminal_rule *next;
} nonterminal_rule;


typedef struct {
  terminal_rule *terminal_rules;
  nonterminal_rule *nonterminal_rules;
} cfg;


typedef struct {
  int len;
  char **tokens; /* array of len tokens */
} sentence;


/* a parse covers part of a sentence using a tree of nonterminals */
typedef struct parse {
  /* this string is owned by the grammar */
  char *nonterminal;
  /* NULL if no further subdivision */
  struct parse *nonterminal_1;
  /* index of last token covered by nonterminal_1 */
  int breakpoint;
  struct parse *nonterminal_2;
  double probability;
  /* in case you want to make a list of them */
  struct parse *next;
} parse;


/* declare functions used outside file where they are defined */

/* grammar.c */
cfg *read_grammar(FILE *infile);
void free_grammar(cfg *g);
void fprint_grammar(FILE *outfile, cfg *g);
sentence *read_sentence(FILE *infile, cfg *g);
void fprint_sentence(FILE *outfile, sentence *s);
void free_sentence(sentence *s);
void fprint_partial_parses(FILE *outfile, parse *ps, sentence *s, int start, int end);
void fprint_parses(FILE *outfile, parse *ps, sentence *s);
void free_parses_shallow(parse *ps);
void free_parses_deep(parse *ps);
/* enum.c */
parse *enum_parse(sentence *s, cfg *g);
/* cyk.c */
parse *cyk_all_parses(sentence *s, cfg *g);
parse *cyk_most_probable_parse(sentence *s, cfg *g);

  
/* EOF */
